@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -13,6 +13,8 @@ import shutil
 import datetime
 import logging
 
+from timestring import Date
+
 from upaas import distro
 
 from upaas import commands
@@ -54,10 +56,13 @@ class BuildResult:
         self.distro_version = distro.distro_version()
         self.distro_arch = distro.distro_arch()
 
+        # information about last commit in VCS (if available)
+        self.vcs_revision = {}
+
 
 class Builder(object):
 
-    #TODO stage decorators
+    # TODO stage decorators
     # @stage('action_system', 95%)
     # time()
 
@@ -73,16 +78,14 @@ class Builder(object):
         """
         self.metadata = metadata
         self.config = builder_config
+        self.envs = {}
+        self.actions = {}
+        self.os_packages = []
 
         self.interpreter_version = utils.select_best_version(self.config,
                                                              metadata)
-        if not self.interpreter_version:
-            self.user_error("Unsupported interpreter version")
 
-        self.actions = self.parse_actions(metadata)
-        self.envs = self.parse_envs(metadata)
-        self.os_packages = self.parse_packages(metadata)
-        self.storage = find_storage_handler(self.config)
+        self.current_revision = None
 
     def user_error(self, msg):
         log.error(msg)
@@ -237,7 +240,7 @@ class Builder(object):
                     ret.add(pkg)
                     log.debug("Will install package '%s' from metadata for "
                               "distribution %s" % (pkg, distro.distro_name()))
-            except KeyError:
+            except (KeyError, TypeError):
                 pass
             try:
                 for pkg in meta.os[distro.distro_name()][
@@ -247,21 +250,39 @@ class Builder(object):
                               "distribution %s, version "
                               "%s" % (pkg, distro.distro_name(),
                                       distro.distro_version()))
-            except KeyError:
+            except (KeyError, TypeError):
                 pass
         return ret
 
-    def build_package(self, system_filename=None):
+    def build_package(self, system_filename=None, interpreter_version=None,
+                      current_revision=None):
         """
         Build a package
 
         :param system_filename: Use given file as base system, if None empty
                                 system image will be used.
+        :param interpreter_version: Use specific interpreter version, only used
+                                    for fresh packages.
+        :param current_revision: VCS revision id from current package.
         """
+        if interpreter_version:
+            self.interpreter_version = interpreter_version
+            log.info("Using forced interpreter version: "
+                     "%s" % interpreter_version)
+
+        self.storage = find_storage_handler(self.config.storage.handler,
+                                            self.config.storage.settings)
         if system_filename and self.storage.exists(system_filename):
             log.info("Starting package build using package "
                      "%s" % system_filename)
+            if current_revision:
+                log.info("VCS revision from current package: "
+                         "%s" % current_revision)
+                self.current_revision = current_revision
         else:
+            if system_filename:
+                log.warning("Requested base package file not found, using "
+                            "empty system image")
             self.envs['UPAAS_FRESH_PACKAGE'] = 'true'
             system_filename = None
             log.info("Starting package build using empty system image")
@@ -274,6 +295,13 @@ class Builder(object):
                     self.system_error("Error during uploading OS image: "
                                       "%s" % e)
 
+        if not self.interpreter_version:
+            self.user_error("Unsupported interpreter version")
+
+        self.actions.update(self.parse_actions(self.metadata))
+        self.envs.update(self.parse_envs(self.metadata))
+        self.os_packages += self.parse_packages(self.metadata)
+
         result = BuildResult()
         result.parent = system_filename
         result.interpreter_version = self.interpreter_version
@@ -319,7 +347,7 @@ class Builder(object):
         result.progress = 40
         yield result
 
-        #TODO if building fails up to this point, then we can try retry it
+        # TODO if building fails up to this point, then we can try retry it
         # on another builder (for a limited number of times)
 
         if system_filename:
@@ -334,11 +362,15 @@ class Builder(object):
         result.progress = 45
         yield result
 
+        result.vcs_revision = self.vcs_info(workdir, chroot_homedir)
+        result.progress = 46
+        yield result
+
         if not self.write_files(workdir, chroot_homedir):
             kill_and_remove_dir(directory)
             self.user_error("Creating files from metadata failed")
         log.info("Created all files from metadata")
-        result.progress = 48
+        result.progress = 49
         yield result
 
         if not self.run_actions(self.app_action_names, workdir,
@@ -399,8 +431,10 @@ class Builder(object):
         yield result
 
     def unpack_os(self, directory, workdir, system_filename=None):
+        empty_os_image = False
         if not system_filename:
             system_filename = distro.distro_image_filename()
+            empty_os_image = True
 
         os_image_path = os.path.join(directory, "os.image")
         log.info("Fetching OS image '%s'" % system_filename)
@@ -414,7 +448,28 @@ class Builder(object):
             if not tar.unpack_tar(os_image_path, workdir):
                 log.error("Error while unpacking OS image to '%s'" % workdir)
                 return False
-        return True
+        # verify if os is working
+        log.info("Checking if OS image is working (will execute /bin/true)")
+        try:
+            with Chroot(workdir):
+                commands.execute('/bin/true',
+                                 timeout=self.config.commands.timelimit,
+                                 output_loglevel=logging.INFO)
+        except Exception as e:
+            log.error("Broken OS image! /bin/true failed: %s" % e)
+            if empty_os_image:
+                try:
+                    self.storage.delete(system_filename)
+                except StorageError as e:
+                    log.error("Storage error while deleting OS image: %s" % e)
+                else:
+                    log.info("Deleted broken OS image from storage")
+                    self.bootstrap_os()
+                    return self.unpack_os(directory, workdir,
+                                          system_filename=system_filename)
+            return False
+        else:
+            return True
 
     def install_packages(self, workdir, packages):
         with Chroot(workdir):
@@ -455,6 +510,61 @@ class Builder(object):
                     return False
         return True
 
+    def vcs_info(self, workdir, homedir):
+        ret = {}
+        log.info("Extracting information about last commit")
+        with Chroot(workdir, workdir=homedir):
+            def vcs_cmd(name, cmd, replace=None):
+                name = 'repository.revision.%s' % name
+                if hasattr(cmd, '__call__'):
+                    cmd = cmd()
+                if replace:
+                    for (rold, rnew) in replace:
+                        cmd = cmd.replace(rold, rnew)
+                env = self.metadata.repository.env.copy()
+                env['LANG'] = 'C.UTF-8'
+                env['LC_ALL'] = 'C.UTF-8'
+                try:
+                    _, output = commands.execute(
+                        cmd, timeout=self.config.commands.timelimit, env=env,
+                        output_loglevel=logging.INFO, strip_envs=True)
+                except commands.CommandTimeout:
+                    log.error("%s command is taking too long, aborting" % name)
+                except commands.CommandFailed:
+                    log.error("%s command failed" % name)
+                else:
+                    return ''.encode('utf-8').join(output).rstrip(
+                        '\n'.encode('utf-8'))
+
+            ret = {
+                'id': vcs_cmd('id', self.metadata.repository.revision.id),
+                'author': vcs_cmd('author',
+                                  self.metadata.repository.revision.author),
+                'date': vcs_cmd('date',
+                                self.metadata.repository.revision.date),
+                'description': vcs_cmd(
+                    'description',
+                    self.metadata.repository.revision.description),
+            }
+
+            if self.current_revision and self.current_revision != ret['id']:
+                ret['changelog'] = vcs_cmd(
+                    'changelog',
+                    self.metadata.repository.revision.changelog,
+                    replace=[
+                        ('%old%', self.current_revision.rstrip('\n')),
+                        ('%new%', ret['id'].rstrip('\n')),
+                    ])
+
+        if 'date' in ret:
+            try:
+                ret['date'] = Date(ret['date']).date
+            except ValueError:
+                log.warning("Can't convert '%s' to date" % ret['date'])
+                del ret['date']
+
+        return ret
+
     def update(self, workdir, homedir):
         log.info("Updating repository in '%s'" % homedir)
         with Chroot(workdir, workdir=homedir):
@@ -528,10 +638,6 @@ class Builder(object):
         """
         Bootstrap base OS image.
         """
-        def _cleanup(directory):
-            log.info("Removing directory '%s'" % directory)
-            shutil.rmtree(directory)
-
         log.info("Bootstrapping new OS image")
 
         # directory is encoded into string to prevent unicode errors
@@ -548,11 +654,11 @@ class Builder(object):
                                  strip_envs=True)
             except commands.CommandTimeout as e:
                 log.error("Bootstrap was taking too long and it was killed")
-                _cleanup(directory)
+                kill_and_remove_dir(directory)
                 raise exceptions.OSBootstrapError(e)
             except commands.CommandFailed as e:
                 log.error("Bootstrap command failed")
-                _cleanup(directory)
+                kill_and_remove_dir(directory)
                 raise exceptions.OSBootstrapError(e)
         log.info("All commands completed, installing packages")
 
@@ -562,7 +668,7 @@ class Builder(object):
         archive_path = os.path.join(directory, "image.tar.gz")
         if not tar.pack_tar(directory, archive_path,
                             timeout=self.config.bootstrap.timelimit):
-            _cleanup(directory)
+            kill_and_remove_dir(directory)
             raise exceptions.OSBootstrapError("Tar error")
         else:
             log.info("Image packed, uploading")
@@ -574,7 +680,7 @@ class Builder(object):
             raise
 
         log.info("Image uploaded")
-        _cleanup(directory)
+        kill_and_remove_dir(directory)
         log.info("All done")
 
     def write_files(self, workdir, chroot_homedir):
@@ -620,4 +726,5 @@ class OSBuilder(Builder):
         :param builder_config: Builder configuration.
         """
         self.config = builder_config
-        self.storage = find_storage_handler(self.config)
+        self.storage = find_storage_handler(self.config.storage.handler,
+                                            self.config.storage.settings)
