@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -124,7 +124,7 @@ def execute(cmd, timeout=None, cwd=None, output_loglevel=logging.DEBUG, env={},
     try:
         while True:
             retcode = p.poll()
-            line = p.stdout.readline()
+            line = p.stdout.readline().decode('utf-8')
             if line:
                 output.append(line)
                 log.log(output_loglevel, line.rstrip(os.linesep))
@@ -140,6 +140,9 @@ def execute(cmd, timeout=None, cwd=None, output_loglevel=logging.DEBUG, env={},
         os.kill(p.pid, signal.SIGKILL)
         _cleanup(wd, original_env)
         raise CommandFailed(e)
+    except Exception:
+        signal.alarm(0)
+        raise
 
     if timeout:
         signal.alarm(0)
