@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -39,7 +39,8 @@ class MongoDBStorage(BaseStorage):
             if self.settings.get('password'):
                 mongouri += ':' + self.settings.password
             mongouri += "@"
-        mongouri += "%s:%s" % (self.settings.host, self.settings.port)
+        mongouri += "%s:%s/%s" % (self.settings.host, self.settings.port,
+                                  self.settings.database)
 
         return MongoClient(mongouri)
 
